MODULE el_spec_plot

use plot_mod

implicit none

!___________________________________________________________________________________________________

INTERFACE
  SUBROUTINE plot_1d_snapshot(ng,x,y,xlab,ylab,title,plot_outfile,pd,time,nt,ymin,ymax,src_x)
  implicit none

  integer ng, pd, pgopen, nt

  real x(1:ng), y(1:ng)
  real, optional :: ymin, ymax, src_x
  character(len=50) :: xlab, ylab, title, time
  character(len=*) :: plot_outfile
  character(len=100) :: plot_dev
  character(len=1) :: q

  END SUBROUTINE
END INTERFACE

INTERFACE
  SUBROUTINE plot_ac_seismogram(t,greens,heavyside,plot_outfile,pd,seismo,analytical,&
                                stf,diff_ana,diff_greens)

  implicit none

  double precision, dimension(:), intent(in) :: t,greens,heavyside
  double precision, dimension(:), intent(in), optional :: seismo,analytical,stf
  double precision, intent(in), optional :: diff_ana,diff_greens
  integer, intent(in) :: pd
  character(len=*) :: plot_outfile
  character(len=100) plot_dev
  character(len=1) dummy

  END SUBROUTINE plot_ac_seismogram
END INTERFACE

INTERFACE
  SUBROUTINE plot_initial(x,y1,y2,plot_outfile,pd)

  implicit none

  double precision, dimension(:), intent(in) :: x,y1,y2
  integer, intent(in) :: pd
  character(len=*), intent(in) :: plot_outfile
  character(len=100) plot_dev
  character(len=1) dummy
  END SUBROUTINE
END INTERFACE

END MODULE el_spec_plot


!___________________________________________________________________________________________________

SUBROUTINE plot_1d_snapshot(ng,x,y,xlab,ylab,title,plot_outfile,pd,time,nt,ymin,ymax,src_x)

!subroutine for plotting simple line graphs during runtime of the program elastic_1D
!with use of the module plot_mod

use plot_mod

implicit none

integer ng, pd, nt, lnw

real x(1:ng), y(1:ng)
real, optional :: ymin, ymax, src_x
character(len=50) :: xlab, ylab, title, time
character(len=*) :: plot_outfile
character(len=100) :: plot_dev

plot_dev=''
time=trim(time)

!if (pgopen('?') .LE. 0) stop

if (pd == 0) then

   plot_dev="/xwindow"

elseif (pd == 1) then

   plot_dev=(plot_outfile)//".gif/gif"
   !if (nt /= 1) call pgpage

else

   plot_dev=trim(plot_outfile)//".ps/cps"

endif

if (pd == 0 .or. pd == 4) then
if (nt == 1) then
   call plot_init(plot_dev,0.0,0.75,1,1,font=1)
endif
else
   call plot_init(plot_dev,0.0,0.75,1,1,font=1)
endif

call pgask(.false.)

call pgbbuf
call pgeras
call pgsch(0.8)
call pgmtxt('T',0.2,0,0.0,time)
call pgsch(1.2)
call pgqlw(lnw)

   call plot_line(y,x,xlab,ylab,title,ymin=ymin,ymax=ymax)
   if(present(src_x)) then
  call pgsci(2)
  call pgslw(2)
  call pgsch(4.0)
  !call pgpt(1,src_x,0.,3)
  call pgpt(1,src_x,0.,ichar('*'))
  call pgsch(1.2)
  call pgslw(lnw)
  call pgsci(1)
   endif

call pgebuf

!call pgclos

if (pd == 1) call pgpage
if (pd == 1) call pgclos

if (pd == 2) call pgpage
if (pd == 2) call pgclos

END SUBROUTINE plot_1d_snapshot


!___________________________________________________________________________________________________

SUBROUTINE plot_ac_seismogram(t,greens,heavyside,plot_outfile,pd,seismo,analytical,&
                              stf,diff_ana,diff_greens)

!subroutine for plotting simple line graphs (seismograms) at the end of the program elastic_1D
!with the use of plot_mod

use plot_mod

implicit none

double precision, dimension(:), intent(in) :: t,greens,heavyside
double precision, dimension(:), intent(in), optional :: seismo,analytical,stf
double precision, intent(in), optional :: diff_ana,diff_greens
double precision diff_a, diff_g
integer, intent(in) :: pd
character(len=*) :: plot_outfile
character(len=30) :: title, title2
character(len=100) plot_dev
character(len=1) dummy

if (.not. present(diff_ana)) then
  diff_a = 0.
else
  diff_a = diff_ana
endif
if (.not. present(diff_greens)) then
  diff_g = 0.
else
  diff_g = diff_greens
endif

if (pd == 0) then
  plot_dev="/xwindow"
elseif (pd == 1) then
  plot_dev=trim(plot_outfile)//'_1'//".gif/gif"
elseif (pd == 2) then
  plot_dev=trim(plot_outfile)//'_1'//".ps/cps"
endif

plot_outfile=trim(plot_outfile)

if (plot_outfile(len_trim(plot_outfile):len_trim(plot_outfile)) == 'u') then
  title='Displacement'
  title2='Greens function - Displacement'
elseif (plot_outfile(len_trim(plot_outfile):len_trim(plot_outfile)) == 'v') then
  title='Velocity'
  title2='Greens function - Velocity'
elseif (plot_outfile(len_trim(plot_outfile):len_trim(plot_outfile)) == 'a') then
  title='Acceleration'
  title2='Greens function - Acceleration'
else
  title='Displacement'
  title2='Greens function - Displacement'
endif


if (present(seismo) .and. present(analytical)) then

  if (present(stf)) then

  call plot_init(plot_dev,0.0,0.75,1,2,font=1)

  call pgpage
  call plot_line(y=real(stf),title='source time function          ')
  call pgpage
  call plot_2lines(x1=real(t+diff_a),y1=real(heavyside),&
  x2=real(t+diff_g),y2=real(greens),title=title2)

  else

  call plot_init(plot_dev,0.0,0.75,1,1,font=1)
  call plot_2lines(x1=real(t+diff_a),y1=real(heavyside),&
  x2=real(t+diff_g),y2=real(greens),title=title2)

  endif

  if (pd == 1 .or. pd == 2) then
  call pgpage
  call pgclos
  if (pd == 1) then
      plot_dev=trim(plot_outfile)//'_2'//".gif/gif"
  elseif (pd == 2) then
      plot_dev=trim(plot_outfile)//'_2'//".ps/cps"
  endif
  endif

  call plot_init(plot_dev,0.0,0.75,1,1,font=1)
  call plot_2lines(real(t+diff_a),real(analytical(1:size(t))),real(t+diff_g),&
                         real(seismo(1:size(t))),title=title)

  call pgebuf

elseif (present(seismo) .and. .not. present(analytical)) then


  if (present(stf)) then

  call plot_init(plot_dev,0.0,0.75,1,2,font=1)

  call pgpage
  call plot_line(y=real(stf),title='source time function          ')
  call pgpage
  call plot_2lines(x1=real(t+diff_a),y1=real(heavyside),&
  x2=real(t+diff_g),y2=real(greens),title=title2)

  else

  call plot_init(plot_dev,0.0,0.75,1,1,font=1)
  call plot_2lines(x1=real(t+diff_a),y1=real(heavyside),&
  x2=real(t+diff_g),y2=real(greens),title=title2)

  endif

  if (pd == 1 .or. pd == 2) then
  call pgpage
  call pgclos
  if (pd == 1) then
      plot_dev=trim(plot_outfile)//'_2'//".gif/gif"
  elseif (pd == 2) then
      plot_dev=trim(plot_outfile)//'_2'//".ps/cps"
  endif
  endif


  call plot_init(plot_dev,0.0,0.75,1,1,font=1)
  call plot_line(real(seismo(1:size(t))),real(t),title=title)

  call pgebuf


elseif (.not. present(seismo) .and. .not. present(analytical)) then

  if (present(stf)) then

  call plot_init(plot_dev,0.0,0.75,1,1,font=1)
  call plot_line(y=real(stf),title='source time function          ')

  endif

  if (pd == 1 .or. pd == 2) then
  if (present(stf)) then
  call pgpage
  call pgclos
  endif
  if (pd == 1) then
      plot_dev=trim(plot_outfile)//'_2'//".gif/gif"
  elseif (pd == 2) then
      plot_dev=trim(plot_outfile)//'_2'//".ps/cps"
  endif
  endif


  call plot_init(plot_dev,0.0,0.75,1,1,font=1)
        call plot_2lines(real(t+diff_a),real(heavyside),real(t+diff_g),real(greens),title=title)

        call pgebuf

  if (pd == 1) call pgpage

else

  write(*,*) "Plotting only the analytical solution doesnt make sense!"


  call plot_init(plot_dev,0.0,0.75,1,1,font=1)
        call plot_2lines(real(t+diff_a),real(heavyside),real(t+diff_g),real(greens),title=title)

        call pgebuf

endif

if (pd == 0 .or. pd == 4) then
  write(*,*) "To close the pgplot window please enter any key"
  read(*,*) dummy
endif



END SUBROUTINE plot_ac_seismogram


!___________________________________________________________________________________________________

SUBROUTINE plot_initial(x,y1,y2,plot_outfile,pd)

!subroutine for plotting the initial wavefield used in the program elastic_1D
!with the use of plot_mod

use plot_mod

implicit none

double precision, dimension(:), intent(in) :: x,y1,y2
integer, intent(in) :: pd
character(len=*), intent(in) :: plot_outfile
character(len=100) plot_dev

if (pd == 0) then
  plot_dev="/xwindow"
elseif (pd == 1) then
  plot_dev=trim(plot_outfile)//".gif/gif"
elseif (pd == 2) then
  plot_dev=trim(plot_outfile)//".ps/cps"
endif


  call plot_init(plot_dev,0.0,0.75,1,2,font=1)

  call pgpage
  call plot_line(y=real(y1),x=real(x),title='Initial Displacement at time 0')
  call pgpage
  call plot_line(y=real(y2),x=real(x),title='Initial Displacement at t=0-dt')

  if (pd == 1 .or. pd == 2) then
  call pgpage
  endif

  call pgebuf

END SUBROUTINE plot_initial


