
module plot_mod

! plot_mod.f90 is a module for plotting and visualizing data sets
  !is using fortran77 pgplot-Graphics Library
  !it has an easy to use user interface where much as possible
  !dummy variables are optional so that the user has not to supply
  !unnessecary variables to have a plotting result as fast as possible
  !written by Gilbert Brietzke
  !Version April 2002

! changed and updated by Bernhard Schuberth 16.01.03 subroutine plot_2lines
! changed and updated by Bernhard Schuberth 28.05.03 subroutine plot_line


  implicit none

  character(len=8):: plot_time, plot_step
  character(len=30):: plot_xlab, plot_ylab, plot_zlab, plot_title
  character(len=100):: plot_parafile, plot_device
  integer :: plot_ipan, plot_jpan
  real :: plot_ratio
  !------------------------------------------------------------------------------------
  interface
     subroutine plot_para(parafile)
       implicit none
       character(len=100) :: parafile
     end subroutine plot_para
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_dots(y,x,xlab,ylab,title,symbol)
       real, dimension(:), intent(in) :: Y
       real, dimension(:), intent(in),optional :: X
       character(len=30),intent(in), optional :: xlab, ylab, title
       integer, intent(in), optional ::  symbol
     end subroutine plot_dots
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_2dots(x1,y1,x2,y2,xlab,ylab,title,symbol1,symbol2,color1,color2)
       integer, intent(in), optional ::  symbol1,symbol2,color1,color2
       real, dimension(:), intent(in) :: Y1,y2
       real, dimension(:), intent(in) :: X1,x2
       character(len=30), optional :: xlab, ylab, title
     end subroutine plot_2dots
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_line(y,x,xlab,ylab,title,xmin,xmax,ymin,ymax)
       real, dimension(:), intent(in) :: Y
       real, dimension(:), intent(in), optional :: X
       real, intent(in), optional :: xmin,xmax,ymin,ymax
       character(len=30),intent(in), optional :: xlab, ylab, title
     end subroutine plot_line
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_2lines(x1,y1,x2,y2,xlab,ylab,title,color1,color2)
       real, dimension(:), intent(in) :: Y1,y2
       real, dimension(:), intent(in) :: X1,x2
       character(len=30), intent(in), optional :: xlab, ylab, title
       integer, intent(in), optional :: color1,color2
     end subroutine plot_2lines
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_3lines(x1,y1,x2,y2,x3,y3,xlab,ylab,title,color1,color2,color3)
       real, dimension(:), intent(in) :: Y1,y2,y3
       real, dimension(:), intent(in) :: X1,x2,x3
       character(len=30), intent(in),optional :: xlab, ylab, title
       integer, intent(in), optional :: color1,color2,color3
     end subroutine plot_3lines
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_multiline(x,y,xlab,ylab,title)
       real, dimension(:,:) :: Y
       real, dimension(:) :: X
       character(len=30), optional :: xlab, ylab, title
     end subroutine plot_multiline
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_init(device,xwinsize,ratio,ipan,jpan,font)
      character(len=100), intent(in) :: device
       real, intent(in), optional :: xwinsize, ratio
       integer, intent(in), optional :: ipan, jpan, font
     end subroutine plot_init
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_image(field,xlab,ylab,zlab,title,colormap,fieldmax,box)
       real, dimension(:,:), intent(in) :: field
       character(len=30), optional :: xlab, ylab, zlab, title
       integer, intent(in), optional :: colormap, box
       real, intent(in), optional :: fieldmax
     END subroutine plot_image
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_vectfield(field1,field2,i1,i2,j1,j2,xlab,ylab,zlab,title,norm)
       real, dimension(:,:), intent(in) :: field1,field2
       integer, intent(in) :: i1,i2,j1,j2
       character(len=30), optional :: xlab, ylab, zlab, title
       integer, intent(in), optional :: norm
     end subroutine plot_vectfield
  end interface
  !__________________________________________________________________
 interface
     SUBROUTINE PALETT(TYPE, CONTRA, BRIGHT)
       INTEGER TYPE
       REAL CONTRA, BRIGHT
     end subroutine PALETT
  end interface
  !__________________________________________________________________
  interface
     SUBROUTINE OUTLIN(I1,I2,J1,J2,TR)
       INTEGER I1,I2,J1,J2
       REAL TR(6)
     end subroutine OUTLIN
  end interface
  interface
     subroutine plot_lun(lun)
       integer :: lun
     end subroutine plot_lun
  end interface
end module plot_mod

!##################################################################################
! module subroutine codes:
!
  subroutine plot_para(parafile)

    implicit none
    !dummy arguments
    character(len=100),intent(in):: parafile
    ! local variables
    integer :: i,n
    real :: x, y, X1, X2, Y1, Y2
    character(len=30):: parastring
    integer :: iun

    call plot_lun(iun)
    n=0
    do i = 1, 1000
      read(iun,'(a30)',end=111) parastring
      n=n+1
    enddo
111 continue
    close(iun)
    open(iun,file=trim(parafile))
    call PGSCF(2)
    CALL PGSCH(.9)
    call pgbbuf
    call pgpanl(1,1)
    call pgeras
    call PGQVP (0, X1, X2, Y1, Y2)
    write(*,*) 'viewport coordinates',x1,x2,y1,y2
    call PGQVSZ(0, X1, X2, Y1, Y2)
    write(*,*) 'viewsurface coordinates',x1,x2,y1,y2
    call PGQWIN (X1, X2, Y1, Y2)
    write(*,*) 'window boundary coordinates',x1,x2,y1,y2
    do i = 1, n
       X = 0.0
       Y = (1.-REAL(i)/real(n))*(y2-y1)
       read(iun,'(a30)') parastring
       parastring = adjustl(trim(parastring))
       write(*,*) X,Y,parastring
       call pgtext(X,Y,parastring)
    enddo
    call pgebuf
    close(iun)

  end subroutine plot_para
!__________________________________________________________________

  subroutine plot_dots(y,x,xlab,ylab,title,symbol)
    ! dummy arguments
    real, dimension(:), intent(in) :: Y
    real, dimension(:), intent(in),optional :: X
    character(len=30),intent(in), optional :: xlab, ylab, title
    integer, intent(in), optional ::  symbol
    !local variables
    real, dimension(:), allocatable :: xtmp
    character(len=30) :: xla, yla, tit
    integer :: i, sym


    CALL PGVSTD
    select case(present(symbol));case(.false.);sym=1 ;case(.true.);sym=symbol;end select
    select case(present(xlab))  ;case(.false.);xla='';case(.true.);xla=xlab  ;end select
    select case(present(ylab))  ;case(.false.);yla='';case(.true.);yla=ylab  ;end select
    select case(present(title)) ;case(.false.);tit='';case(.true.);tit=title ;end select
    call pglab(xla,yla,tit)
    if (present(x)) then
      CALL PGSWIN (minval(x),maxval(x),minval(y),maxval(y))
      CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
      call pgpt(size(y),x,y,sym)
    else
      allocate(xtmp(1:size(y)))
      do i = 1, size(y)
         xtmp(i) = real(i)
      enddo
      CALL PGSWIN (minval(xtmp),maxval(xtmp),minval(y),maxval(y))
      CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
      call pgpt(size(y),xtmp,y,sym)
       deallocate(xtmp)
    endif

  end subroutine plot_dots
!__________________________________________________________________
  subroutine plot_2dots(x1,y1,x2,y2,xlab,ylab,title,symbol1,symbol2,color1,color2)
    ! dummy arguments
    integer, intent(in), optional ::  symbol1,symbol2,color1,color2
    real, dimension(:), intent(in) :: Y1,y2
    real, dimension(:), intent(in) :: X1,x2
    character(len=30), optional, intent(in) :: xlab, ylab, title
    ! local variables
    character(len=30) :: xla, yla, tit
    real :: minx,maxx,miny,maxy
    integer :: color
!    CALL PGPAGE
    minx=minval(min(x1,x2))
    miny=minval(min(y1,y2))
    maxx=maxval(max(x1,x2))
    maxy=maxval(max(y1,y2))

    CALL PGVSTD
    CALL PGSWIN (minx,maxx,miny,maxy)
    CALL PGBOX ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select
    call pglab(xla,yla,tit)
    call pgqci(color)
    call pgsci(color1)
    call pgpt(size(y1),x1,y1,symbol1)
    call pgsci(color2)
    call pgpt(size(y2),x2,y2,symbol2)
    call pgsci(color)
  end subroutine plot_2dots
!__________________________________________________________________

! subroutine changed by Bernhard Schuberth 28.05.03
! now it is possible to give minimum and maximum values of x and y to be plotted

  subroutine plot_line(y,x,xlab,ylab,title,xmin,xmax,ymin,ymax)
    ! dummy arguments
    real, dimension(:),intent(in) :: y
    real, dimension(:), intent(in), optional :: x
    real, intent(in), optional :: xmin,xmax,ymin,ymax
    character(len=30),intent(in), optional :: xlab, ylab, title
    ! internal arguments
    real, dimension(:), allocatable :: xtmp
    real minx,maxx,miny,maxy
    character(len=30) :: xla, yla, tit
    integer i

    CALL PGVSTD
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select
    call pglab(xla,yla,tit)

    select case(present(ymin));case(.false.);miny=minval(y);case(.true.);miny=ymin;end select
    select case(present(ymax));case(.false.);maxy=maxval(y);case(.true.);maxy=ymax;end select

    if (present(x)) then

      select case(present(xmin));case(.false.);minx=minval(x);case(.true.);minx=xmin;end select
      select case(present(xmax));case(.false.);maxx=maxval(x);case(.true.);maxx=xmax;end select

      CALL PGSWIN (minx,maxx,miny,maxy)
      CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
      call pgline(size(y),x,y)
    else
      allocate(xtmp(1:size(y)))
      do i = 1,size(y)
         xtmp(i) = real(i)
      enddo
    CALL PGSWIN (minval(xtmp),maxval(xtmp),miny,maxy)
    CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
    call pgline(size(y),xtmp,y)
    deallocate(xtmp)
    endif

 end subroutine plot_line
!__________________________________________________________________

  subroutine plot_2lines(x1,y1,x2,y2,xlab,ylab,title,color1,color2)
    ! dummy arguments
    real, dimension(:), intent(in) :: Y1,y2
    real, dimension(:), intent(in) :: X1,x2
    character(len=30), optional, intent(in) :: xlab, ylab, title
    integer, intent(in), optional ::color1, color2
    ! local variables
    character(len=30) :: xla, yla, tit
    real :: minx,maxx,miny,maxy
    integer color

!Changed by Bernhard Schuberth 16.01.2003
    minx=min(minval(x1),minval(x2))
    miny=min(minval(y1),minval(y2))
    maxx=max(maxval(x1),maxval(x2))
    maxy=max(maxval(y1),maxval(y2))
!Old Version
!    minx=minval(min(x1,x2))
!    miny=minval(min(y1,y2))
!    maxx=maxval(max(x1,x2))
!    maxy=maxval(max(y1,y2))

    CALL PGVSTD
    CALL PGSWIN (minx,maxx,miny,maxy)
    CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select
    call pglab(xla,yla,tit)
    call pgqci(color)
    if (present(color1))then
       call pgsci(color1)
    else
       call pgsci(1)
    endif
    call pgline(size(y1),x1,y1)
    if (present(color2))then
       call pgsci(color2)
    else
       call pgsci(2)
    endif
    call pgline(size(y2),x2,y2)
    call pgsci(color)

  end subroutine plot_2lines
!__________________________________________________________________

  subroutine plot_3lines(x1,y1,x2,y2,x3,y3,xlab,ylab,title,color1,color2,color3)
    ! dummy arguments
    real, dimension(:), intent(in) :: Y1,y2,y3
    real, dimension(:), intent(in) :: X1,x2,x3
    character(len=30), intent(in), optional :: xlab, ylab, title
    integer, intent(in), optional :: color1, color2, color3
    ! local variables
    character(len=30):: xla, yla, tit
    real :: minx,maxx,miny,maxy
    integer:: color

    minx=minval(min(x1,x2,x3))
    miny=minval(min(y1,y2,y3))
    maxx=maxval(max(x1,x2,x3))
    maxy=maxval(max(y1,y2,y3))
!    if(abs(maxx-minx)<=tiny

    CALL PGVSTD
    CALL PGSWIN (minx,maxx,miny,maxy)
    CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select
    call pglab(xla,yla,tit)
    call pgqci(color)
    if (present(color1))then
       call pgsci(color1)
    else
       call pgsci(1)!'white'
    endif
    call pgline(size(y1),x1,y1)
    if (present(color2))then
       call pgsci(color2)
    else
       call pgsci(2)!'red'
    endif
    call pgline(size(y2),x2,y2)
    if (present(color3))then
       call pgsci(color3)
    else
       call pgsci(3)!'green' blue:4
    endif
    call pgline(size(y3),x3,y3)
    call pgsci(color)

  end subroutine plot_3lines
!__________________________________________________________________
  subroutine plot_multiline(x,y,xlab,ylab,title)

    integer :: I, pgopen, symbol
    real, dimension(:,:) :: Y
    real, dimension(:) :: X
    character(len=30),intent(in), optional :: xlab, ylab, title
    ! local variables
    character(len=30):: xla, yla, tit

!    call pgunsa
!    call pgvstd
    call pgsvp(0.15,0.85,0.15,0.85)
    call pgswin(MINVAL(X)-(Maxval(X)-MINVAL(X))/50.,Maxval(X)+(Maxval(X)-MINVAL(X))/50. &
              &,MINVAL(Y)-(MAXVAL(Y)-MINVAL(Y))/5.,MAXVAL(Y)+(MAXVAL(Y)-MINVAL(Y))/5.)
!    call pgsave
    call PGSCI(1)
    call PGSCI(14)
    call PGBOX('G',0.,0,'G',0.0,0)
    call PGSCI(1)
    CALL PGBOX('ABCTSN',0.0,0,'ABCTSNV',0.0,0)
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select
    call pglab(xla,yla,tit)
!    call PGSCI(48)
    call PGSCI(46)
    do i = 1, size(y,2)
       call pgline(size(y,1),x,y(:,i))
    enddo
    call PGSCI(14)
    call PGBOX('G',0.,0,'G',0.0,0)
    call PGSCI(1)
    call PGBOX('ABCTSN',0.0,0,'ABCTSNV',0.0,0)

  end subroutine plot_multiline


!__________________________________________________________________


subroutine plot_init(device,xwinsize,ratio,ipan,jpan,font)
  ! dummy variables
    character(len=100), intent(in) :: device
    real, intent(in), optional :: xwinsize, ratio
    integer, intent(in), optional :: ipan, jpan, font
    ! inner variables
    integer :: pgopen, L, C1, C2, NC,ci1,ci2, istat
    real :: contra, bright, c, s
    character(len=16) :: VAL

    ISTAT = PGOPEN(trim(device))
!    write(*,*) 'pgopen: Istat =', Istat
    IF (ISTAT .LE. 0 ) then
       ISTAT = PGOPEN('?')
    endif
    IF (ISTAT .LE. 0 ) stop

    CALL PGQINF('TYPE', VAL, L)
    CALL PGQCIR(C1, C2)
    call PGQCOL (CI1, CI2)
    NC = MAX(0, C2-C1+1)
    IF (NC .LT.8) THEN
       WRITE (*,*) 'Not enough colors available on this device'
       STOP
    END IF
    if ((present(ipan)).and.(present(jpan))) then
       call pgsubp(ipan,jpan)
    else
       call pgsubp(1,1)
    endif
    if ((present(xwinsize)).and.(present(ratio))) then
       call PGPAP(xwinsize,ratio)
!       write(*,*) ' set xwinsize & ratio '
   elseif((present(xwinsize)).and..not.(present(ratio)))then
       call PGPAP(xwinsize,1.)
    else
       call PGPAP(10.,1.)
    endif
    if (present(font)) call PGSCF (font)
! Set the Character Font for subsequent text plotting.
! 1: (default) a simple single-stroke font, 2: roman font, 3: italic font,  4: script font
    call pgsch(2.)
!    call pgsave

end subroutine plot_init
!__________________________________________________________________

subroutine plot_vectfield(field1,field2,i1,i2,j1,j2,xlab,ylab,zlab,title,norm)

  implicit none

  real, dimension(:,:), intent(in) :: field1,field2
  integer, intent(in) :: i1,i2,j1,j2
  character(len=30), optional :: xlab, ylab, zlab, title
  integer, intent(in), optional :: norm

  integer :: MAXX, MAXY
  real :: TR(6), tempch

  MAXX = size(field1,1)
  MAXY = size(field1,2)
  TR(1) = 1.0; TR(2) = 1.0
  TR(3) = 0.0; TR(4) = 1.0
  TR(5) = 0.0; TR(6) = 1.0
  CALL PGSVP (0.15, 0.85, 0.15, 0.85)
  CALL PGWNAD(real(i1), real(i2), real(j1), real(j2))
  CALL PGBOX('bcntsi',0.0,0,'bcntsiv',0.0,0)
  CALL PGLAB (xlab,ylab,title)
  CALL PGSAH (2, 45.0, 0.7)
  CALL PGQCH (TEMPCH)
  CALL PGSCH (0.3)
  CALL PGVECT (field1,field2,maxx,maxy,i1,i2,j1,j2,0.0,0,TR,1.e31)
  CALL PGSCH (TEMPCH)

end subroutine plot_vectfield

!__________________________________________________________________

  subroutine plot_image(field,xlab,ylab,zlab,title,colormap,fieldmax,box)
    ! dummy variables
    real, dimension(:,:), intent(in) :: field
    character(len=30), optional :: xlab, ylab, zlab, title
    integer, intent(in), optional :: colormap,box
    real, intent(in), optional :: fieldmax
    ! inner variables
    character(len=30):: xla, yla, tit
    integer :: mxi, mxj, L, C1, C2, NC, idefault, ifont
    real :: fmin, fmax, contra, bright, angle, c, s
    real, dimension(1:6) :: TR(6)
    character(len=16) :: VAL
    character(len=10), parameter :: routine_name = 'plot_image'
    real :: x1,x2,y1,y2

    select case(present(xlab));case(.false.) ;xla='';case(.true.);xla=xlab;end select
    select case(present(ylab));case(.false.) ;yla='';case(.true.);yla=ylab;end select
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select

    mxi = size(field,1)
    mxj = size(field,2)
    if (present(fieldmax)) then
       fmin = - fieldmax
       fmax = fieldmax
       else
          fmax =  maxval(abs(field))
          fmin = -fmax
    endif

    TR(1) = 0.0
    TR(2) = 1.0
    TR(3) = 0.0
    TR(4) = 0.0
    TR(5) = 0.0
    TR(6) = 1.0

    if (mxi<mxj) then
       CALL PGSVP (0.15, 0.89, 0.15, 0.85)
    else
       CALL PGSVP (0.15, 0.89, 0.25, 0.85)
    endif
    CALL PGWNAD(0.0, 1.0+MXI, 0.0, 1.0+MXJ)
    if (present(colormap)) then
       BRIGHT = 0.50
       CONTRA  = 1.0
       CALL PALETT(colormap, CONTRA, BRIGHT)
    else
       BRIGHT = 0.50
       CONTRA  = 1.0
       idefault = 6
       CALL PALETT(idefault, CONTRA, BRIGHT)
    endif

    if (abs(fmin)>1.e6*tiny(1.)) then
       CALL PGIMAG(Field,MXI,MXJ,1,MXI,1,MXJ,FMIN,FMAX,TR)
    endif
!    call PGQWIn(X1, X2, Y1, Y2)
!    call pgcirc((x2-x1)*0.5,(y2-y1)*0.5,min((y2-y1)*0.25,(x2-x1)*0.25))
    if(present(title)) then
       tit=title
       CALL PGMTXT('t',1.0,0.5,0.5,title)
    else
       tit=' '
    endif
    if(present(box)) then
  if(box==1)then; CALL PGBOX('bcntsi',0.0,0,'bcntsiv',0.0,0)
        elseif(box==0)then
        endif
    else
  CALL PGBOX('bcntsi',0.0,0,'bcntsiv',0.0,0)
    endif
    if (abs(fmin)>1.e6*tiny(1.)) then
       if (mxi<mxj) then
          if(present(zlab))then
             CALL PGWEDG('RI', 0.75, 3., FMIN, FMAX, zlab)
          else
             CALL PGWEDG('RI', 0.75, 3., FMIN, FMAX,'')
          endif
       else
          if(present(zlab))then
             CALL PGWEDG('BI', 2., 3., FMIN, FMAX, zlab)
          else
             CALL PGWEDG('BI', 2., 3., FMIN, FMAX,'')
          endif
       endif
    endif

  end subroutine plot_image

!--------------------------------------------------------------------

  SUBROUTINE PALETT(TYPE, CONTRA, BRIGHT)
!C-----------------------------------------------------------------------
!C Set a "palette" of colors in the range of color indices used by
!C PGIMAG.
!C-----------------------------------------------------------------------
      INTEGER TYPE
      REAL CONTRA, BRIGHT
!C
      REAL GL(2), GR(2), GG(2), GB(2)
      REAL GiL(2), GiR(2), GiG(2), GiB(2)
      REAL sL(3), sR(3), sG(3), sB(3)
      REAL xL(4), xR(4), xG(4), xB(4)
      REAL RL(9), RR(9), RG(9), RB(9)
      REAL HL(5), HR(5), HG(5), HB(5)
      REAL WL(10), WR(10), WG(10), WB(10)
      REAL AL(20), AR(20), AG(20), AB(20)
!C
      DATA GL /0.0, 1.0/
      DATA GR /0.0, 1.0/
      DATA GG /0.0, 1.0/
      DATA GB /0.0, 1.0/
!c
      DATA GiL /0.0, 1.0/
      DATA GiR /1.0, 0.0/
      DATA GiG /1.0, 0.0/
      DATA GiB /1.0, 0.0/
!c
      DATA sL /0.0, 0.5, 1.0/
      DATA sR /1.0, 0.9, 0.0/
      DATA sG /0.0, 1.0, 0.0/
      DATA sB /0.0, 0.9, 1.0/

      DATA xL /0.0, 0.51, 0.51 , 1.0/
      DATA xR /0.0, 0.0, 0.0 , 0.0/
      DATA xG /0.0, 0.0, 0.8 , 0.8/
      DATA xB /0.0, 0.0, 0.0 , 0.0/

!      DATA sL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
!      DATA sR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
!      DATA sG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
!      DATA sB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
!C
      DATA RL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
      DATA RR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
      DATA RG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
      DATA RB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
!C
!      DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
!      DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
!      DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
!      DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/

      DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
      DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
      DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
      DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/
!C
      DATA WL /0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, 0.95, 1.0/
      DATA WR /0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0,  1.0, 1.0/
      DATA WG /0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  1.0, 1.0/
      DATA WB /0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, 0.95, 1.0/
!C
      DATA AL /0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5,&
     &         0.5, 0.6, 0.6, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0/
      DATA AR /0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0,&
     &         0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0/
      DATA AG /0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8,&
     &         0.6, 0.6, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0/
      DATA AB /0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9,&
     &         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/
!C
      IF (TYPE.EQ.1) THEN
!C        -- gray scale
         CALL PGCTAB(GL, GR, GG, GB, 2, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.2) THEN
!C        -- rainbow
         CALL PGCTAB(RL, RR, RG, RB, 9, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.3) THEN
!C        -- heat
         CALL PGCTAB(HL, HR, HG, HB, 5, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.4) THEN
!C        -- weird IRAF
         CALL PGCTAB(WL, WR, WG, WB, 10, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.5) THEN
!C        -- AIPS
         CALL PGCTAB(AL, AR, AG, AB, 20, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.6) THEN
!C        -- AIPS
         CALL PGCTAB(sL, sR, sG, sB, 5, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.7) THEN
!C        -- AIPS
         CALL PGCTAB(xL, xR, xG, xB, 5, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.8) THEN
!C        -- AIPS
         CALL PGCTAB(GiL, GiR, GiG, GiB, 2, CONTRA, BRIGHT)
      END IF
      END subroutine

      SUBROUTINE SETVP
!C-----------------------------------------------------------------------
!C Set the viewport, allowing margins around the edge for annotation.
!C (This is similar in effect to PGVSTD, but has different margins.)
!C The routine determines the view-surface size and allocates margins
!C as fractions of the minimum of width and height.
!C-----------------------------------------------------------------------
      REAL D, VPX1, VPX2, VPY1, VPY2
!C
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 8.0*D
      VPY2 = VPY2 - 2.0*D
      CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2)
      END subroutine

      SUBROUTINE FIDDLE
!C
      INTEGER P, IER, PGCURS
      REAL CONTRA, BRIGHT, X, Y, SIGN
      REAL X1, Y1, X2, Y2, B1, B2, C1, C2
      CHARACTER CH
!C
      WRITE (*,*) 'Use cursor to adjust color table:'
      WRITE (*,*) ' Keys 1,2,3,4,5 select different palettes'
      WRITE (*,*) ' Key P cycles through available palettes'
      WRITE (*,*) ' Key F adjusts contrast and brightness, with'
      WRITE (*,*) '  cursor x position setting brightness [0.0 - 1.0]'
      WRITE (*,*) '   and y position setting contrast [0.0 - 10.0]'
      WRITE (*,*) '  (Hold down F key while moving cursor to change'
      WRITE (*,*) '  contrast and brightness continuously)'
      WRITE (*,*) ' Key C resets contrast=1.0, brightness=0.5'
      WRITE (*,*) ' Key - reverses color palette'
      WRITE (*,*) ' Key X or right mouse button exits program'
!C
      P = 2
      CONTRA = 1.0
      BRIGHT = 0.5
      X = 0.5
      Y = 1.0
      SIGN = +1.0
!C
      CALL PGQWIN(X1, X2, Y1, Y2)
      B1 = 0.0
      B2 = 1.0
      C1 = 0.0
      C2 = 10.0
      CALL PGSWIN(B1, B2, C1, C2)
 10   IER = PGCURS(X, Y, CH)
      IF (CH.EQ.CHAR(0) .OR. CH.EQ.'x' .OR. CH.EQ.'X') THEN
         CALL PGSWIN(X1, X2, Y1, Y2)
         RETURN
      ELSE IF (CH.EQ.'F' .OR. CH.EQ.'f') THEN
         BRIGHT = MAX(B1, MIN(B2,X))
         CONTRA = MAX(C1, MIN(C2,Y))
      ELSE IF (CH.EQ.'C' .OR. CH.EQ.'c') THEN
         CONTRA = 1.0
         Y = 1.0
         BRIGHT = 0.5
         X = 0.5
      ELSE IF (CH.EQ.'-') THEN
         SIGN = -SIGN
      ELSE IF (CH.EQ.'1') THEN
         P = 1
      ELSE IF (CH.EQ.'2') THEN
         P = 2
      ELSE IF (CH.EQ.'3') THEN
         P = 3
      ELSE IF (CH.EQ.'4') THEN
         P = 4
      ELSE IF (CH.EQ.'5') THEN
         P = 5
      ELSE IF (CH.EQ.'P' .OR. CH.EQ.'p') THEN
         P = 1 + MOD(P,5)
      END IF
      CALL PALETT(P, SIGN*CONTRA, BRIGHT)
      GOTO 10
      END subroutine

      SUBROUTINE OUTLIN(I1,I2,J1,J2,TR)
      INTEGER I1,I2,J1,J2
      REAL TR(6)
!C-----------------------------------------------------------------------
!C Draw the enclosing rectangle of the subarray to be contoured,
!C applying the transformation TR.
!C
!C For a contour map, the corners are (I1,J1) and (I2,J2); for
!C a gray-scale map, they are (I1-0.5,J1-0.5), (I2+0.5, J2+0.5).
!C-----------------------------------------------------------------------
      INTEGER K
      REAL XW(5), YW(5), T
!C
      XW(1) = I1
      YW(1) = J1
      XW(2) = I1
      YW(2) = J2
      XW(3) = I2
      YW(3) = J2
      XW(4) = I2
      YW(4) = J1
      XW(5) = I1
      YW(5) = J1
      DO 10 K=1,5
          T = XW(K)
          XW(K) = TR(1) + TR(2)*T + TR(3)*YW(K)
          YW(K) = TR(4) + TR(5)*T + TR(6)*YW(K)
   10 CONTINUE
      CALL PGLINE(5,XW,YW)

    end subroutine

!---------------------------------------------------------------------------
subroutine plot_lun(lun)
  implicit none
  ! subroutine plot_lun written by Gilbert Brietzke 26.11.2001 for plot_mod -module
  ! original version by Dates D. Boore 05/19/98, following Larry Baker's suggestion
  ! dummy variables:
  integer :: lun
  !local variables
  logical :: isopen
  integer :: i
  do i = 99,10,-1
     inquire (unit=i, opened=isopen)
     if(.not.isopen) then
        lun = i
        return
     endif
  enddo
  lun = -1
  return
end subroutine plot_lun

!***<contour>***********************************************************
!***<some untility routines>********************************************
!
SUBROUTINE VCOPY(X,Y,N)
  !     -----------------------
  !
  REAL X(*),Y(*)
  !
  DO I=1,N
     Y(I)=X(I)
  ENDdo
end SUBROUTINE VCOPY
!
SUBROUTINE VRFILL(X,A,N)
  !     ------------------------
  REAL X(*)
  !
  DO I=1,N
     X(I)=A
  END DO
end SUBROUTINE VRFILL
!
SUBROUTINE LOGQYN(S,D,L)
  !     ------------------------
  !
  LOGICAL      L,L2
  CHARACTER(len=1) D,D2,A
  CHARACTER(len=45) STRING
  CHARACTER(len=*) S
  !
  IF (D.EQ.'Y') THEN
     L=.TRUE.
     D2='N'
     L2=.FALSE.
  ELSEIF (D.EQ.'N') THEN
     L=.FALSE.
     D2='Y'
     L2=.TRUE.
  ELSE
     WRITE(*,*)' Default should be Y or N !'
     RETURN
  ENDIF
  CALL STPACK(STRING,S,45)
1 WRITE(*,100) STRING,D
100 FORMAT(A45,' Y/N (def=',A1,')  : ')
  CALL FORMTQ(A,1,NNN)
  IF (NNN.EQ.0) RETURN
  IF (A.EQ.'y' .OR. A.EQ.'T' .OR. A.EQ.'t') A='Y'
  IF (A.EQ.'n' .OR. A.EQ.'F' .OR. A.EQ.'f') A='N'
  IF (A.EQ.D) THEN
     RETURN
  ELSEIF (A.EQ.D2) THEN
     L=L2
     RETURN
  ENDIF
  GOTO 1
END SUBROUTINE LOGQYN
!
SUBROUTINE STPACK(STRING,S,N)
  !     -----------------------------
  !
  CHARACTER STRING(*),S(*)
  !
  DO I=1,N
     STRING(I)=S(I)
     IF (S(I).EQ.'?') GOTO 20
  enddo
20 continue
  DO J=I+1,N
     STRING(J)=' '
  enddo
END SUBROUTINE STPACK
!
SUBROUTINE FORMTQ(STRING,NCHAR,NNN)
  ! -----------------------------------
  CHARACTER(len=*) STRING
  !
  READ(*,200) STRING
200 FORMAT(A)
  NNN=0
  DO I=1,NCHAR
     IF (STRING(I:I).NE.' ') NNN=NNN+1
  ENDDO
end SUBROUTINE FORMTQ
!C
      SUBROUTINE FILARY(F,NX,NY,NZ,FMIN,FMAX)
!C     ---------------------------------------
!C
      REAL F(-NX:NX,-NY:NY,-NZ:NZ)
      DATA SMALL,RMAX,THREE /1.0E-10,10.0,3.0/
!C
      FMAX=0.0
      FMIN=1.0E20
      DX=RMAX/FLOAT(NX)
      DY=RMAX/FLOAT(NY)
      DZ=RMAX/FLOAT(NZ)
      Z=-RMAX
      DO K=-NZ,NZ
        ZZ=Z**2
        Y=-RMAX
        DO J=-NY,NY
          YY=Y**2
          X=-RMAX
          DO I=-NX,NX
            RR=X**2+YY+ZZ+SMALL
            R=SQRT(RR)
            PSI2=EXP(-R)*(THREE*ZZ-RR)**2
            F(I,J,K)=PSI2
            FMAX=MAX(PSI2,FMAX)
            FMIN=MIN(PSI2,FMIN)
            X=X+DX
         enddo
          Y=Y+DY
       enddo
        Z=Z+DZ
     enddo
   END SUBROUTINE FILARY
!C
      SUBROUTINE AXINIT(A,B,C,ALFA,BETA,GAMA,LATICE,RADANG,ZSHIFT)
!C     ------------------------------------------------------------
!C
      REAL LATICE(3,4,*)
!C
      ALF=RADANG*ALFA
      BET=RADANG*BETA
      GAM=RADANG*GAMA
      AX=A
      AY=0.0
      AZ=0.0
      BX=B*COS(GAM)
      BY=B*SIN(GAM)
      BZ=0.0
      CX=C*COS(BET)
      CY=(C*COS(ALF)-CX*COS(GAM))/SIN(GAM)
      CZ=SQRT(C**2-CX**2-CY**2)
      XSHIFT=-0.5*(AX+BX+CX)
      YSHIFT=-0.5*(AY+BY+CY)
      ZSHIFT=-0.5*(AZ+BZ+CZ)
      LATICE(1,1,2)=XSHIFT
      LATICE(2,1,2)=YSHIFT
      LATICE(3,1,2)=ZSHIFT
      LATICE(1,2,2)=AX+XSHIFT
      LATICE(2,2,2)=AY+YSHIFT
      LATICE(3,2,2)=AZ+ZSHIFT
      LATICE(1,3,2)=BX+XSHIFT
      LATICE(2,3,2)=BY+YSHIFT
      LATICE(3,3,2)=BZ+ZSHIFT
      LATICE(1,4,2)=CX+XSHIFT
      LATICE(2,4,2)=CY+YSHIFT
      LATICE(3,4,2)=CZ+ZSHIFT
      ZSHIFT=-MAX(A,B,C)
    END SUBROUTINE AXINIT
!C
      SUBROUTINE VIEW3A(JPHI1,JTHT,JPHI2,LATICE,RADANG,ZSHIFT)
!C     --------------------------------------------------------
!C
      REAL      LATICE(3,4,*)
      CHARACTER STRING*60
!C
!      WRITE(*,*)
!   1  WRITE(*,100) JPHI1,JTHT,JPHI2
! 100  FORMAT(' View> Euler angles ?  (def=',I2,',',I2,',',I2,
!     *       ' deg) : ',$)
!      CALL FORMTQ(STRING,60,NNN)
!      IF (NNN.NE.0) THEN
!        READ(STRING,*,ERR=1) IPHI1,ITHT,IPHI2!
!        IF (ITHT.LT.0 .OR. ITHT.GT.180) GOTO 1
!        IF (IPHI1.LT.0 .OR. IPHI1.GT.360) GOTO 1
!        IF (IPHI2.LT.0 .OR. IPHI2.GT.360) GOTO 1
!        JTHT=ITHT
!        JPHI1=IPHI1
!        JPHI2=IPHI2
!      ENDIF
!      jphi1=0
!      jphi2=45
!      jtht=90
      THT=FLOAT(JTHT)*RADANG
      PHI1=FLOAT(JPHI1)*RADANG
      PHI2=FLOAT(JPHI2)*RADANG
      SINTHT=-SIN(THT)
      COSTHT=COS(THT)
      SINPH1=-SIN(PHI1)
      COSPH1=COS(PHI1)
      SINPH2=SIN(PHI2)
      COSPH2=COS(PHI2)
      DO J=1,4
        CALL ROTZ(LATICE(1,J,2),LATICE(2,J,2),LATICE(3,J,2),U,V,W,SINPH2,COSPH2)
        CALL ROTY(U,V,W,X,Y,Z,SINTHT,COSTHT)
        CALL ROTZ(X,Y,Z,LATICE(1,J,1),LATICE(2,J,1),LATICE(3,J,1),SINPH1,COSPH1)
        LATICE(3,J,1)=LATICE(3,J,1)+ZSHIFT
     enddo
   END SUBROUTINE VIEW3A
!C
      SUBROUTINE ROTY(X,Y,Z,U,V,W,S,C)
!C     --------------------------------
      U=X*C-Z*S
      V=Y
      W=X*S+Z*C
    END SUBROUTINE ROTY
!C
      SUBROUTINE ROTZ(X,Y,Z,U,V,W,S,C)
!C     --------------------------------
!C
      U=X*C+Y*S
      V=-X*S+Y*C
      W=Z
    END SUBROUTINE ROTZ
!C
      SUBROUTINE VIEW3C(LATICE,SLNORM,APOINT)
!C     ---------------------------------------
!C
      REAL LATICE(3,*),SLNORM(*),APOINT(*)
!C
      DO I=1,3
        APOINT(I)=0.5*(LATICE(I,4)+LATICE(I,3)+LATICE(I,2)-LATICE(I,1))
        SLNORM(I)=LATICE(I,2)-LATICE(I,1)
     enddo
   END SUBROUTINE VIEW3C

   SUBROUTINE MYVIEWxy(LATICE,SLNORM,APOINT)
!C     ---------------------------------------------
!C
      REAL LATICE(3,*),SLNORM(*),APOINT(*)
!C
      DO I=1,3
        APOINT(I)=0.5*(LATICE(I,4)+LATICE(I,3)+LATICE(I,2)-LATICE(I,1))
        SLNORM(I)=LATICE(I,4)-LATICE(I,1)
     ENDDO
   END SUBROUTINE MYVIEWxy
!C
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
   SUBROUTINE MYVIEWxz(LATICE,SLNORM,APOINT)
!C     ---------------------------------------------
!C
      REAL LATICE(3,*),SLNORM(*),APOINT(*)
!C
      DO I=1,3
        APOINT(I)=0.5*(LATICE(I,4)+LATICE(I,3)+LATICE(I,2)-LATICE(I,1))
        SLNORM(I)=LATICE(I,3)-LATICE(I,1)
     ENDDO
   END SUBROUTINE MYVIEWxz
!C
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE myview1(LATICE,SLNORM,APOINT,SFRAC)
!C     ---------------------------------------------
!C
      REAL LATICE(3,*),SLNORM(*),APOINT(*)
!C
      DO I=1,3
        SLNORM(I)=LATICE(I,2)-LATICE(I,1)
        APOINT(I)=0.5*(LATICE(I,4)+LATICE(I,3))+SFRAC*SLNORM(I)
      enddo
    END SUBROUTINE myview1
!C
!C!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE myslice(LATICE,SLNORM,APOINT,SFRAC,dir)
!C     ---------------------------------------------
!C
      REAL LATICE(3,*),SLNORM(*),APOINT(*)
      character(len=2)::dir
!C
      if(dir=='xy')then
         DO I=1,3
            SLNORM(I)=LATICE(I,4)-LATICE(I,1)
            APOINT(I)=0.5*(LATICE(I,3)+LATICE(I,2))+SFRAC*SLNORM(I)
         enddo
      elseif(dir=='xz')then
         DO I=1,3
            SLNORM(I)=LATICE(I,3)-LATICE(I,1)
            APOINT(I)=0.5*(LATICE(I,4)+LATICE(I,2))+SFRAC*SLNORM(I)
         enddo
      elseif(dir=='yz')then
         DO I=1,3
            SLNORM(I)=LATICE(I,2)-LATICE(I,1)
            APOINT(I)=0.5*(LATICE(I,4)+LATICE(I,3))+SFRAC*SLNORM(I)
         enddo
      endif

    END SUBROUTINE myslice
!C
!C!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


!C!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE myviewcontrol(LATICE,position,SLNORM,APOINT)
!C     ---------------------------------------------
!C
      REAL :: LATICE(3,*),SLNORM(*),APOINT(*), position

!C
      DO I=1,3
        APOINT(I)=position*(LATICE(I,4)+LATICE(I,3)+LATICE(I,2)-LATICE(I,1))
        SLNORM(I)=LATICE(I,4)-LATICE(I,1)
      enddo
    END SUBROUTINE myviewcontrol
!C
    !C
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!C modified by Gilbert Brietzke 30.11.2001
!c
!C     ------------------------------
      SUBROUTINE STCOL1(LUT,R,G,B,N)
!C     ------------------------------
!C
      REAL LUT(3,*),R(*),G(*),B(*)
!C
      DO I=1,N
        R(I)=LUT(1,I)
        G(I)=LUT(2,I)
        B(I)=LUT(3,I)
     enddo
   END SUBROUTINE STCOL1

!C
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!
   ! for reference : Bronstein Ausgabe 2 S.197-198 Eulersche Winkel

   subroutine euler_rota(v,theta,psi,phi,vtrans)

     implicit none

     real, dimension(3),intent(in) :: v
     real, dimension(3),intent(out) ::vtrans
     real, intent(in) :: theta,psi,phi

     real, parameter :: pi=3.1415926535898
     real :: l1,l2,l3,m1,m2,m3,n1,n2,n3
     real :: c1,c2,c3,s1,s2,s3

!      theta,psi,phi
     c1=cos(theta*pi/180.); c2=cos(psi*pi/180.); c3=cos(phi*pi/180.)
     s1=sin(theta*pi/180.); s2=sin(psi*pi/180.); s3=sin(phi*pi/180.)

     l1 = c2*c3 - c1*s2*s3
     l2 = -c2*s3 - c1*s2*c3
     l3 = s1*s2

     m1 = s2*c3 + c1*s2*s3
     m2 = -s2*s3 + c1*c2*c3
     m3 = -s1*c2

     n1 = s1*s3
     n2 = s1*c3
     n3 = c1

     vtrans(1)=l1*v(1)+m1*v(2)+n1*v(3)
     vtrans(2)=l2*v(1)+m2*v(2)+n2*v(3)
     vtrans(3)=l3*v(1)+m3*v(2)+n3*v(3)
!     write(*,*) l1,l2,l3
!     write(*,*) m1,m2,m3
!     write(*,*) n1,n2,n3
!     write(*,*) c1,c2,c3
!     write(*,*) s1,s2,s3
!     write(*,*) vtrans

   end subroutine euler_rota
