
module sem_analyt_mod

implicit none

contains

subroutine calcanalyt_sem(analytical,seismopos,dt,dx,sourcex,rhomodel,mumodel,source,sctype,diff_anas,rectimes)

  integer,dimension(:)::seismopos
  double precision,dimension(:)::source,rhomodel,mumodel,dx,diff_anas,rectimes
  double precision,dimension(size(seismopos),size(source))::analytical
  double precision::dt
  integer::sourcex,nt,sctype,irec

  write(*,*) ' starting calculation of analyticals '
  analytical=0.
  nt=size(source)

  select case(sctype)
  case(0) ! initial velocity at time 0 (not 1 !)

  do irec=1,size(analytical,1)
     rectimes(irec)=rectime_sem(mumodel,rhomodel,sourcex,seismopos(irec),dx)
     diff_anas(irec)=rectimes(irec)-dble(floor(rectimes(irec)/dt))*dt
     analytical(irec,(-floor(-rectimes(irec)/dt))+1:nt)=1.
  end do

  end select

  write(*,*) ' analyticals calculated, leaving module'

end subroutine

! the following calculates the two layer case
subroutine calcanalyt2lay_sem(analytical,seismopos,dt,dx,sourcex,rhomodel,mumodel,source,modpar,&
            diff_anas,rectimes,reftimes,rec_dx)

  integer,dimension(:)::seismopos
  double precision,dimension(:)::source,rhomodel,mumodel,dx,diff_anas,rectimes,reftimes
  double precision,dimension(size(seismopos),size(source))::analytical
  double precision, dimension(:), optional :: rec_dx
  double precision::dt,refamp,refzerotime
  integer::sourcex,nt,irec,modpar,it,rt,rft

  write(*,*) ' starting calculation of analyticals 2 layer case'
  analytical=0.
  nt=size(source)

  ! refzerotime is the time the signal reaches the discontinuity
  refzerotime=rectime_sem(mumodel,rhomodel,sourcex,modpar,dx)

! modpar is the index of the discontunity
  if(modpar < 2) modpar = 2
  refamp=abs(sqrt(mumodel(modpar-1)*rhomodel(modpar-1))-sqrt(mumodel(modpar+1)*rhomodel(modpar+1)))&
  & /(sqrt(mumodel(modpar-1)*rhomodel(modpar-1))+sqrt(mumodel(modpar+1)*rhomodel(modpar+1)))

  do irec=1,size(analytical,1)
     if (present(rec_dx)) then
     rectimes(irec)=rectime_sem(mumodel,rhomodel,sourcex,seismopos(irec),dx,rec_dx(irec))
     reftimes(irec)=reftime_sem(mumodel,rhomodel,modpar,seismopos(irec),dx,rec_dx(irec))+refzerotime
     else
     rectimes(irec)=rectime_sem(mumodel,rhomodel,sourcex,seismopos(irec),dx)
     reftimes(irec)=reftime_sem(mumodel,rhomodel,modpar,seismopos(irec),dx)+refzerotime
     endif
     diff_anas(irec)=rectimes(irec)-dble(floor(rectimes(irec)/dt))*dt
     rt=-floor(-rectimes(irec)/dt)+1
     rft=-floor(-reftimes(irec)/dt)+1
     do it=rt,nt
        analytical(irec,it)=dble(1.)
     end do
  ! calculate reflection coefficient
     do it=rft,nt
       if (sqrt(mumodel(1)/rhomodel(1)) < sqrt(mumodel(size(mumodel))/rhomodel(size(mumodel)))) then
         analytical(irec,it)=dble(1.)-refamp
       else
         analytical(irec,it)=dble(1.)+refamp
       endif
     end do
  end do

  write(*,*) ' analyticals calculated, leaving module'

end subroutine

function rectime_sem(mumodel,rhomodel,srcx,recx,dx,rec_dx)

  double precision,dimension(:)::rhomodel,mumodel,dx
  double precision,dimension(size(rhomodel))::vmodel
  double precision, optional :: rec_dx
  integer::srcx,recx,ix, i
  double precision::rectime_sem

  rectime_sem=0.
  do i=1,size(rhomodel)
  vmodel(i)=sqrt(mumodel(i)/rhomodel(i))
  enddo

  ! advice dx always as dx between current and next point
  do ix=1,abs(srcx-recx)-1
   rectime_sem=rectime_sem+dx(srcx+ix)/vmodel(srcx+ix)
  end do

  if (present(rec_dx)) rectime_sem = rectime_sem + rec_dx/vmodel(recx)

end function

function reftime_sem(mumodel,rhomodel,modpar,recx,dx,rec_dx)

  double precision,dimension(:)::rhomodel,mumodel,dx
  double precision,dimension(size(rhomodel))::vmodel
  double precision, optional :: rec_dx
  integer::modpar,recx,ix, i
  double precision::reftime_sem

  reftime_sem=0.
  do i=1,size(rhomodel)
  vmodel(i)=sqrt(mumodel(i)/rhomodel(i))
  enddo

  ! advice dx always as dx between current and next point
  if (modpar > recx ) then
     do ix=1,(modpar-recx)-1
        reftime_sem=reftime_sem+dx(recx-1+ix)/vmodel(recx-1+ix)
     end do
     if (present(rec_dx)) reftime_sem = reftime_sem - rec_dx/vmodel(recx)
  else
     do ix=1,(recx-modpar)
        reftime_sem=reftime_sem+dx(modpar-1+ix)/vmodel(modpar-1+ix)
     end do
     if (present(rec_dx)) reftime_sem = reftime_sem + rec_dx/vmodel(recx)
  endif

end function

end module sem_analyt_mod
