MODULE sem_spec_modules


implicit none

!******************************************************************************

INTERFACE
  SUBROUTINE diff_lagrange(N,DL,rho,xi)

    Implicit none

    integer N,i,j,k

    double precision xi(0:N),LL(0:N),VN(0:N),rho(0:N)
    double precision DL(0:N,0:N)

  END SUBROUTINE
END INTERFACE

INTERFACE
  SUBROUTINE create_x_irreg(x,xmesh,shape1D)

  implicit none

  double precision, dimension(:,:) :: shape1D
  double precision, dimension(:) :: x, xmesh
  integer N,ne,ng,NGNOD, ispec,i,j
  END SUBROUTINE
END INTERFACE

INTERFACE
  SUBROUTINE connect_vec(N,ne,ng,el,GL,C)

    integer N,ne,ng
    integer i,j,k

    double precision el(1:N+1,1:ne), GL(1:ng)

    integer C(1:N+1,1:ne)

  END SUBROUTINE
END INTERFACE

INTERFACE
  SUBROUTINE connect_mat(N,ne,ng,el,GL,C)

    integer N,ne,ng
    integer i,j,k

    double precision el(1:N+1,1:N+1,1:ne), GL(1:ng,1:ng)

    integer C(1:N+1,1:ne)

  END SUBROUTINE
END INTERFACE

INTERFACE
  SUBROUTINE calculate_F(N,ne,DL,weights,lambdae,U,F,C,Jacobian,Jacobi_inverse,ve_src,src,src_ne,src_N,bv1,bvng,stfpar,dt)

  implicit none

  integer i,j,k,l, ispec, N, ne, src_ne, src_N

  double precision, dimension(1:N+1,1:ne) :: lambdae
  double precision, dimension(1:N+1,1:N+1) :: DL
  double precision, dimension(:,:) :: Jacobian, Jacobi_inverse
  double precision, dimension(:) :: U, F
  double precision, dimension(1:N+1) :: fe, weights, tmpx1

  double precision src, ve_src, sigma, tmp
  double precision, optional :: bv1, bvng, dt

  integer, dimension(1:N+1,1:ne) :: C

  character(len=2), optional :: stfpar

  END SUBROUTINE
END INTERFACE

INTERFACE
  SUBROUTINE get_func_at_pos(N,num_el,func,outval,x,xigll,C,x_anchor)

  implicit none

  integer i,j,k, N,num_el
  double precision, dimension(:),intent(in) :: func,x_anchor
  double precision, intent(in) :: x
  double precision, dimension(1:N+1) :: l,dl,xigll
  double precision outval, xi_local

  integer, dimension(:,:),intent(in) :: C
  END SUBROUTINE
END INTERFACE

END MODULE sem_spec_modules


!______________________________________________________________________________

SUBROUTINE diff_lagrange(N,DL,rho,xi)
!
!calculates the derivatives of the Lagrange Polynomials at the GLL quadrature
!points and stores them in Matrix DL (coloumn = d (L)/d (xi) = dL, row = dL at
!point xi

Implicit none

integer N,i
double precision xi(0:N),LL(0:N),VN(0:N),rho(0:N)
double precision DL(0:N,0:N)

!Calculate the Gau-Lobatto-Legendre Quadrature points and the values of the
!Legendre polynomials VN at the gll nodes

call main_pol(xi,N,VN)

!calculate the weights rho(i) at every gll node for the gll integration rule

rho = 2 / ( float((N+1)*N) * VN**2)

!calculating the derivatives of the Polynomials at the  Gau-Lobatto-Legendre Quadrature points

do i=0,N

    LL=0
    LL(i)=1
    call DELEGL(N,xi,VN,LL,DL(:,i))

enddo

END SUBROUTINE diff_lagrange

!_____________________________________________________________________________
SUBROUTINE main_pol(xi,N,VN)

IMPLICIT NONE

INTEGER j,N
DOUBLE PRECISION X,xi(0:N),ET(0:1000),VN(0:1000)

DO j=0,1000
X=-1+REAL(j)/1000.*2
CALL ZELEGL(N,ET,VN)
ENDDO
DO j=0,N

  xi(j)=ET(j)

ENDDO

END SUBROUTINE main_pol


!____________________________________________________________________________
      SUBROUTINE VALEPO(N,X,Y,DY,D2Y)

!*   COMPUTES THE VALUE OF THE LEGENDRE POLYNOMIAL OF DEGREE N
!*   AND ITS FIRST AND SECOND DERIVATIVES AT A GIVEN POINT
!*   N  = DEGREE OF THE POLYNOMIAL
!*   X  = POINT IN WHICH THE COMPUTATION IS PERFORMED
!*   Y  = VALUE OF THE POLYNOMIAL IN X
!*   DY = VALUE OF THE FIRST DERIVATIVE IN X
!*   D2Y= VALUE OF THE SECOND DERIVATIVE IN X

      IMPLICIT NONE

      DOUBLE PRECISION Y,DY,D2Y,X,YP,DYP,D2YP,C1,C2,C4,YM,DYM,D2YM
      INTEGER N,I

         Y   = 1.
         DY  = 0.
         D2Y = 0.

      IF (N .EQ. 0) RETURN

         Y   = X
         DY  = 1.
         D2Y = 0.

      IF(N .EQ. 1) RETURN

         YP   = 1.
         DYP  = 0.
         D2YP = 0.

      DO I=2,N
         C1 = FLOAT(I)
         C2 = 2.*C1-1.
         C4 = C1-1.
         YM = Y
         Y  = (C2*X*Y-C4*YP)/C1
         YP = YM
         DYM  = DY
         DY   = (C2*X*DY-C4*DYP+C2*YP)/C1
         DYP  = DYM
         D2YM = D2Y
         D2Y  = (C2*X*D2Y-C4*D2YP+2.D0*C2*DYP)/C1
         D2YP = D2YM
       ENDDO

      RETURN

END SUBROUTINE VALEPO


!____________________________________________________________________________

      SUBROUTINE ZELEGL(N,ET,VN)

!*   COMPUTES THE NODES RELATIVE TO THE LEGENDRE GAUSS-LOBATTO FORMULA
!*   N  = ORDER OF THE FORMULA
!*   ET = VECTOR OF THE NODES, ET(I), I=0,N
!*   VN = VALUES OF THE LEGENDRE POLYNOMIAL AT THE NODES, VN(I), I=0,N

      IMPLICIT NONE

      DOUBLE PRECISION SN,X,Y,DY,D2Y,PI,C,ETX
      DOUBLE PRECISION ET(0:1000),VN(0:1000)
      INTEGER N,N2,I,IT

!     DIMENSION ET(0:*), VN(0:*)

      IF (N .EQ. 0) RETURN

         N2 = (N-1)/2
         SN = FLOAT(2*N-4*N2-3)
         ET(0) = -1.
         ET(N) = 1.
         VN(0) = SN
         VN(N) = 1.
      IF (N .EQ. 1) RETURN

         ET(N2+1) = 0.
         X = 0.
      CALL VALEPO(N,X,Y,DY,D2Y)
         VN(N2+1) = Y
      IF(N .EQ. 2) RETURN

         PI = 3.14159265358979323846
         C  = PI/FLOAT(N)
      DO I=1,N2
        ETX = DCOS(C*FLOAT(I))
        DO IT=1,8
         CALL VALEPO(N,ETX,Y,DY,D2Y)
         ETX = ETX-DY/D2Y
        ENDDO
        ET(I) = -ETX
        ET(N-I) = ETX
        VN(I) = Y*SN
        VN(N-I) = Y
      ENDDO

      RETURN
      END SUBROUTINE ZELEGL


!_______________________________________________________________________

      SUBROUTINE DELEGL(N,ET,VN,QN,DQN)

!************************************************************************
!*  COMPUTES THE DERIVATIVE OF A POLYNOMIAL AT THE LEGENDRE GAUSS-LOBATTO
!*  NODES FROM THE VALUES OF THE POLYNOMIAL ATTAINED AT THE SAME POINTS
!*   N   = THE DEGREE OF THE POLYNOMIAL
!*   ET  = VECTOR OF THE NODES, ET(I), I=0,N
!*   VN  = VALUES OF THE LEGENDRE POLYNOMIAL AT THE NODES, VN(I), I=0,N
!*   QN  = VALUES OF THE POLYNOMIAL AT THE NODES, QN(I), I=0,N
!*   DQN = DERIVATIVES OF THE POLYNOMIAL AT THE NODES, DQZ(I), I=0,N
!************************************************************************

  implicit none

  integer n,i,j
  double precision su,vi,ei,vj,ej,dn,c

  double precision ET(0:*), VN(0:*), QN(0:*), DQN(0:*)

          DQN(0) = 0.D0
      IF (N .EQ. 0) RETURN

      DO 1 I=0,N
          SU = 0.D0
          VI = VN(I)
          EI = ET(I)
      DO 2 J=0,N
      IF (I .EQ. J) GOTO 2
          VJ = VN(J)
          EJ = ET(J)
          SU = SU+QN(J)/(VJ*(EI-EJ))
2     CONTINUE
          DQN(I) = VI*SU
1     CONTINUE

          DN = FLOAT(N)
          C  = .25D0*DN*(DN+1.D0)
          DQN(0) = DQN(0)-C*QN(0)
          DQN(N) = DQN(N)+C*QN(N)

      END SUBROUTINE DELEGL

!_____________________________________________________________________________
!_____________________________________________________________________________

SUBROUTINE create_x_irreg(x,xmesh,shape1D)

!creates a non equidistant x vector of length ng corresponding to the gll nodes
!in each element and depending on the x-vector xmesh.

implicit none

double precision, dimension(:,:) :: shape1D
double precision, dimension(:) :: x, xmesh
integer N,ne,ng,NGNOD, ispec,i,j

N=size(shape1D,2)-1
ne=size(xmesh)-1
ng=size(x)
NGNOD=size(shape1D,1)


do ispec=1,ne
  do i=1,N
    do j=1,NGNOD
  x((ispec-1)*N+i) = x((ispec-1)*N+i) + shape1D(j,i)*xmesh(ispec-1+j)
    enddo
  enddo
enddo

do j=1,NGNOD
  x(ng) = x(ng) + shape1D(j,N+1)*xmesh(ne-1+j)
enddo

END SUBROUTINE create_x_irreg

!______________________________________________________________________________
!______________________________________________________________________________

SUBROUTINE connect_vec(N,ne,ng,el,GL,C)

!subroutine for compiling a global vector GL (adding the components el(i,j) at the correct places)

integer N,ne,ng
integer i,j

double precision el(1:N+1,1:ne), GL(1:ng)

integer C(1:N+1,1:ne)

do i=1,ne

    do j=1,N+1

        GL(C(j,i))=GL(C(j,i))+el(j,i)

    enddo

enddo

END SUBROUTINE connect_vec


!______________________________________________________________________________
!______________________________________________________________________________

SUBROUTINE connect_mat(N,ne,ng,el,GL,C)

!subroutine for compiling a global matrix GL (adding the components el(i,j) at the correct places)

integer N,ne,ng
integer i,j,k

double precision el(1:N+1,1:N+1,1:ne), GL(1:ng,1:ng)

integer C(1:N+1,1:ne)

do i=1,ne

    do j=1,N+1

        do k=1,N+1

            GL(C(k,i),C(j,i))=GL(C(k,i),C(j,i))+el(k,j,i)

        enddo

    enddo

enddo

END SUBROUTINE connect_mat

!__________________________________________________________________________________

SUBROUTINE calculate_F(N,ne,DL,weights,lambdae,U,F,C,Jacobian,Jacobi_inverse,ve_src,src,src_ne,src_N,bv1,bvng,stfpar,dt)

implicit none

integer i,l, ispec, N, ne, src_ne, src_N

double precision src, ve_src, sigma, tmp
double precision, optional :: bv1, bvng, dt

double precision, dimension(1:N+1,1:ne) :: lambdae
double precision, dimension(1:N+1,1:N+1) :: DL
double precision, dimension(:,:) :: Jacobian, Jacobi_inverse
double precision, dimension(:) :: U, F
double precision, dimension(1:N+1) :: fe, weights, tmpx1

integer, dimension(1:N+1,1:ne) :: C

character(len=2), optional :: stfpar

do ispec=1,ne

    fe=0

    do i=1,N+1
  tmp = 0.
      do l=1,N+1

  tmp= tmp + U(C(l,ispec)) * DL(i,l)  !calculating du/dxi = epsilon in 1D
      enddo

  ! calculating the stress in physical coordinates => multiply with dxi/dx = Jacobi_inverse
  sigma = lambdae(i,ispec) * tmp * Jacobi_inverse(i,ispec)

  ! Jacobian needed for integration and second dxi/dx needed for derivation of
  !test function in physical coordinates
  tmpx1(i) = Jacobian(i,ispec) * sigma * Jacobi_inverse(i,ispec)
    enddo

    do i = 1,N+1

  tmp = 0.

  do l=1,N+1

     tmp = tmp + tmpx1(l) * DL(l,i) * weights(l)
  enddo

  fe(i) = - tmp

  if (present(bv1) .and. present(bvng)) then
     if (i == 1 .and. ispec == 1) fe(i)=fe(i)-bv1
     if (i == (N+1) .and. ispec == ne) fe(i)=fe(i)-bvng
  endif

  if (ispec == src_ne .and. i == src_N) then
    if ((.not. (present(stfpar) .or. present(dt))) .or. stfpar == 'de') then
     fe(i) = fe(i) + 2. * src * ve_src
    else
     fe(i) = fe(i) + 2. * src * ve_src / dt
    endif
  endif

    enddo

    do i = 1,N+1
        F(C(i,ispec))=F(C(i,ispec))+fe(i)
    enddo

enddo


END SUBROUTINE calculate_F


!____________________________________________________________________________________

SUBROUTINE get_func_at_pos(N,num_el,func,outval,x,xigll,C,x_anchor)

use lagrange_poly

implicit none

integer i,N,num_el

double precision, dimension(:),intent(in) :: func,x_anchor
double precision, intent(in) :: x
double precision, dimension(1:N+1) :: l,dl,xigll
double precision outval, xi_local

integer, dimension(:,:),intent(in) :: C

xi_local = 0.
outval = 0.

xi_local = -1. + (x - x_anchor(num_el))/(x_anchor(num_el+1) - x_anchor(num_el))*2.

call lagrange_any(xi_local,N+1,xigll,l,dl)

do i=1,N+1
  outval = outval + func(C(i,num_el)) * l(i)
enddo


END SUBROUTINE get_func_at_pos

