MODULE shape_mod

! Changed to 1D by Bernhard Schuberth 02.06.2003 for serial applications

!=====================================================================
!
!          S p e c f e m 3 D  G l o b e  V e r s i o n  3 . 3
!          --------------------------------------------------
!
!                 Dimitri Komatitsch and Jeroen Tromp
!    Seismological Laboratory - California Institute of Technology
!        (c) California Institute of Technology September 2002
!
!    A signed non-commercial agreement is required to use this program.
!   Please check http://www.gps.caltech.edu/research/jtromp for details.
!           Free for non-commercial academic research ONLY.
!      This program is distributed WITHOUT ANY WARRANTY whatsoever.
!      Do not redistribute this program without written permission.
!
!=====================================================================

INTERFACE
  SUBROUTINE get_shape1D(shape1D,dershape1D,xigll)

  implicit none

    double precision xigll(:)
    double precision shape1D(:,:)
    double precision dershape1D(:,:)

    integer i,j,k,ia, NGLLX, NGNOD

    double precision xi
    double precision l1xi,l2xi,l3xi
    double precision l1pxi,l2pxi,l3pxi

    double precision sumshape,sumdershapexi
    double precision, parameter :: TINYVAL=1e-9

    double precision, parameter :: ONE = 1.
    double precision, parameter :: HALF = 1./2.
    double precision, parameter :: ZERO = 0.
    double precision, parameter :: TWO = 2.
  END SUBROUTINE
END INTERFACE

END MODULE


!__________________________________________________________________

  subroutine get_shape1D(shape1D,dershape1D,xigll)

  implicit none

! Gauss-Lobatto-Legendre points of integration
  double precision xigll(:)

! 1D shape functions and their derivatives
  double precision shape1D(:,:)
  double precision dershape1D(:,:)

  integer i,ia, NGLLX, NGNOD

! location of the nodes of the 1D quadrilateral elements
  double precision xi
  double precision l1xi,l2xi,l3xi
  double precision l1pxi,l2pxi,l3pxi

! for checking the 1D shape functions
  double precision sumshape,sumdershapexi
  double precision, parameter :: TINYVAL=1e-9

! defining 1, 1/2 and 0 and 2

  double precision, parameter :: ONE = 1.
  double precision, parameter :: HALF = 1./2.
  double precision, parameter :: ZERO = 0.
  double precision, parameter :: TWO = 2.

NGNOD = size(shape1D,1)
NGLLX = size(shape1D,2)


if (NGNOD == 2) then

! generate the 1D shape functions and their derivatives (2 nodes)

  do i=1,NGLLX

  xi=xigll(i)

  l1xi=(ONE - xi) * HALF
  l2xi=(ONE + xi) * HALF

  l1pxi=-HALF
  l2pxi=HALF

!     corner nodes

      shape1D(1,i)=l1xi
      shape1D(2,i)=l2xi

      dershape1D(1,i)=l1pxi
      dershape1D(2,i)=l2pxi

  enddo

elseif (NGNOD == 3) then

! generate the 1D shape functions and their derivatives (3 nodes)

  do i=1,NGLLX

  xi=xigll(i)

  l1xi=HALF*xi*(xi-ONE)
  l2xi=ONE-xi**2
  l3xi=HALF*xi*(xi+ONE)

  l1pxi=xi-HALF
  l2pxi=-TWO*xi
  l3pxi=xi+HALF


!     corner nodes

      shape1D(1,i)=l1xi
      shape1D(2,i)=l2xi
      shape1D(3,i)=l3xi

      dershape1D(1,i)=l1pxi
      dershape1D(2,i)=l2pxi
      dershape1D(3,i)=l3pxi

  enddo

endif

! check the shape functions
  do i=1,NGLLX

      sumshape=ZERO

      sumdershapexi=ZERO

      do ia=1,NGNOD

        sumshape=sumshape+shape1D(ia,i)

        sumdershapexi=sumdershapexi+dershape1D(ia,i)

      enddo

!     the sum of the shape functions should be 1
      if(abs(sumshape-ONE) > TINYVAL) print *, 'error in 1D shape functions'

!     the sum of the derivatives of the shape functions should be 0
      if(abs(sumdershapexi) > TINYVAL) &
        print *, 'error in xi derivatives of 1D shape function'

  enddo

  end subroutine get_shape1D

