MODULE source

!******************************************************************************

INTERFACE
  SUBROUTINE ricker2(out2,dt,pt,nt)
   implicit none

   double precision   dt, pt
   integer i,nt
   double precision, dimension(1:(nt+1)) :: ricker1
   double precision, dimension(:) :: out2
  END SUBROUTINE ricker2
END INTERFACE

INTERFACE
  SUBROUTINE ricker(out1,dt,pt,mt)
   implicit none

   double precision dt, pt, t0, a_ricker, t
   integer it,mt
   double precision, dimension(1:mt) :: out1
  END SUBROUTINE ricker
END INTERFACE

INTERFACE
  SUBROUTINE delta(nt,ntpeak,dt,src)
   implicit none

   double precision, dimension(:) :: src
   double precision dt
   integer  nt, ntpeak
  END SUBROUTINE delta
END INTERFACE

INTERFACE
  SUBROUTINE create_heavyside(heavyside,nt,dt,distance,v)

  implicit none

  double precision, dimension(:), intent(inout) :: heavyside
  double precision, intent(in) :: dt, distance,v

  integer, intent(in) :: nt
  integer i

  END SUBROUTINE create_heavyside
END INTERFACE
INTERFACE
  SUBROUTINE gauss(output,dt,pt)

  implicit none

  double precision, dimension(:), intent(inout) :: output
  double precision, dimension(:), allocatable :: tmp
  double precision dt, pt

  integer i, mt
  END SUBROUTINE
END INTERFACE

END MODULE source


!******************************************************************************

SUBROUTINE ricker2(out2,dt,pt,nt)

!creates a source time function that has always the same shape of a ricker 2 wavelet
!i.e. it is the derivate of a ricker wavelet, which is the same as the second derivative
!of a gaussian funcion
!in the intervall [0,tmax] for all orders N of Lagrangian interpolants
!and for all numbers ne of elements

use stringfuns_tbib

implicit none

double precision   dt, pt
integer i, nt
double precision, dimension(1:(nt+1)) :: ricker1
double precision, dimension(:) :: out2

!defining the source time function

  !allocating all vectors, which length is dependent on nt

     call ricker(ricker1,dt,pt,(nt+1))

  out2=0

  do i=1,nt

     out2(i) = ricker1(i+1) - ricker1(i)

  enddo


END SUBROUTINE ricker2


!******************************************************************************

SUBROUTINE ricker(out1,dt,pt,mt)
! RICKER generate a ricker wavelet
! input (outputvector,dt,period,number of time steps)

implicit none

double precision dt, pt, t0, a_ricker, t
integer it,mt
double precision, dimension(1:mt) :: out1

  out1=0

  t0=pt/dt
  a_ricker=4./pt

  do it=1,mt
    t=(it-t0)*dt
    out1(it) = -2.*a_ricker*t*exp(-(a_ricker*t)**2)
  enddo

END SUBROUTINE ricker


!************************************************************************************

SUBROUTINE delta(nt,ntpeak,dt,src)

!creates a delta source time function with the delta peak at timestep ntpeak
!in the intervall [0,tmax] for all orders N of Lagrangian interpolants
!and for all numbers ne of elements

implicit none

double precision, dimension(:) :: src
double precision dt
integer  nt, ntpeak


!defining the source time function

src=0

if (ntpeak < nt) then
      src(ntpeak) = 1./dt
else

   write(*,*) "The time step ntpeak at which the delta peak shall occur must be smaller than nt"
   write(*,*) "nt =", nt
   stop

endif


END SUBROUTINE delta


!******************************************************************************

SUBROUTINE create_heavyside(heavyside,nt,dt,distance,v)

implicit none

double precision, dimension(:), intent(inout) :: heavyside
double precision, intent(in) :: dt, distance,v

integer, intent(in) :: nt
integer i

heavyside = 0

heavyside=(/ (0.,i=1,idint((distance)/(v*dt))-1),&
(1.,i=idint((distance)/(v*dt)),nt) /)


END SUBROUTINE create_heavyside


!******************************************************************************

SUBROUTINE gauss(output,dt,pt)

implicit none

double precision, dimension(:), intent(inout) :: output
double precision, dimension(:), allocatable :: tmp
double precision dt, pt

integer i, mt

mt=size(output)

allocate(tmp(1:mt-1))

call ricker(tmp,dt,pt,mt-1)

output(1) = tmp(1)

do i=2,mt

  output(i) = output(i-1) + tmp(i-1)

enddo

END SUBROUTINE
