MODULE write_data

use stringfuns_tbib

implicit none

!******************************************************************************

INTERFACE
  SUBROUTINE create_sem_fname(name1,begstr,N,nletters,ne,neletters,receiver_id)
    implicit none

    integer, intent(in) :: N,ne,nletters,neletters
    integer, intent(in) :: receiver_id
    character(len=50), intent(inout) :: name1
    character(len=*) begstr
    character(len=nletters) nstr
    character(len=neletters) nestr
  END SUBROUTINE

END INTERFACE

INTERFACE
  SUBROUTINE write_seismograms(filename,seismogram,analytical,srd,receiver_nr,rec_nrletters,dngix,diff_seismo,diff_ana,seismo_id)

    implicit none

    double precision,dimension(:), intent(in) :: seismogram,analytical
    double precision, intent(in) :: srd
    double precision, intent(in), optional :: diff_seismo,diff_ana

    integer, intent(in) :: receiver_nr,rec_nrletters,dngix
    integer i,nt,na

    character(len=50), intent(inout) :: filename
    character(len=50) outfile1,outfile2,outfile3
    character(len=rec_nrletters) rec_nrstr
    character(len=1), intent(in), optional :: seismo_id
  END SUBROUTINE

END INTERFACE

INTERFACE
  SUBROUTINE write_setup(filename,N,ne,mod_id,receiver_id,dt,nt,Length,min_v,tmax,dx_mean,ng,rec_num,source_ix,de)

    implicit none

    double precision, intent(in) :: dt,Length,min_v,tmax,dx_mean
    double precision, optional :: de

    integer, intent(in) :: N,ne,nt,ng,rec_num,source_ix

    character(len=*),intent(in) :: filename
    character(len=*), intent(in) :: mod_id
    integer, intent(in) :: receiver_id
  END SUBROUTINE
END INTERFACE

END MODULE write_data


!******************************************************************************

SUBROUTINE create_sem_fname(name1,begstr,N,nletters,ne,neletters,receiver_id)

use stringfuns_tbib

!creates a string (filename) "ac_s" + (nletters-log(N))0 N "_" + (neletters-log(ne))0 ne +
!+ "_" + receiver_id + "r"

implicit none

integer, intent(in) :: N,ne,nletters,neletters
integer, intent(in) :: receiver_id
character(len=50), intent(inout) :: name1
character(len=*) begstr
character(len=nletters) nstr
character(len=neletters) nestr

nstr = nletterstring(N,nletters)

nestr = nletterstring(ne,neletters)

if(receiver_id == 0) then
  name1=trim(begstr)//nstr//"_"//nestr//"_0r"
else
  name1=trim(begstr)//nstr//"_"//nestr//"_1r"
endif

END SUBROUTINE create_sem_fname


!******************************************************************************

SUBROUTINE write_seismograms(filename,seismogram,analytical,srd,receiver_nr,rec_nrletters,dngix,diff_seismo,diff_ana,seismo_id)

use stringfuns_tbib

!creates three output files


implicit none

double precision,dimension(:), intent(in) :: seismogram,analytical
double precision, intent(in) :: srd
double precision, intent(in), optional :: diff_seismo,diff_ana

integer, intent(in) :: receiver_nr,rec_nrletters,dngix
integer i,nt,na

character(len=50), intent(inout) :: filename
character(len=50) outfile1,outfile2,outfile3

character(len=rec_nrletters) rec_nrstr
character(len=1), intent(in), optional :: seismo_id


nt = size(seismogram)
na = size(analytical)

rec_nrstr = nletterstring(receiver_nr,rec_nrletters)

!creating output file 1 "filename + receiver_nr .dat"

  if (present(seismo_id)) then
      outfile1 = 'DATA_TMP/'//trim(filename)//trim(rec_nrstr)//'_'//seismo_id//".dat"
  else
      outfile1 = 'DATA_TMP/'//trim(filename)//trim(rec_nrstr)//".dat"
  endif
    open (unit=16,file=outfile1,form='formatted')


!creating output file 2 "filename + receiver_nr _analyt.dat"

  outfile2 = 'DATA_TMP/'//trim(filename)//trim(rec_nrstr)//"_analyt.dat"
    open (unit=15,file=outfile2,form='formatted')

!creating output file 3 "filename + receiver_nr _info.m"

    outfile3 = 'DATA_TMP/'//trim(filename)//trim(rec_nrstr)//"_info.m"
    open (unit=17,file=outfile3,form='formatted')

!write Greens function of model at receiver(j) (at distance(j)) to output file 1

    do i=1,nt
        write(16,*) seismogram(i)
    enddo

!write heavysidefunction to output file 2

    do i=1,na
        write(15,*) analytical(i)
    enddo

!write additional data to  output file 3

    write(17,*) 'srd=', srd , "% exact_source_receiver_distance"
    write(17,*) 'dngx=', dngix , "% number of nodes from source to receiver"

  if (present(diff_ana)) then

      write(17,*) 'diff_ana=', diff_ana , "% theoretical_time_difference_between"
      write(17,*) "% analytical_solution_and_discrete_analytical_solution"
  endif

  if (present(diff_seismo)) then

      write(17,*) 'diff_seismo=', diff_seismo , "% theoretical_time"
      write(17,*) "% _error_of_seismogram"
  endif

    close(16)
    close(15)
    close(17)


END SUBROUTINE

!**************************************************************************************

SUBROUTINE write_setup(filename,N,ne,mod_id,receiver_id,dt,nt,Length,min_v,tmax,dx_mean,ng,rec_num,source_ix,de)

implicit none

double precision, intent(in) :: dt,Length,min_v,tmax,dx_mean
double precision, intent(in), optional :: de

integer, intent(in) :: N,ne,nt,ng,rec_num,source_ix

character(len=*), intent(in) :: filename
character(len=*), intent(in) :: mod_id
integer, intent(in) :: receiver_id

open(unit=18,file=filename,form='formatted')

    write(18,*) 'N=', N , "% order_of_Lagrange_polynomials"
    write(18,*) 'NE=', ne , "%number_of_elements"
    write(18,*) 'OPID=''', trim(mod_id) , "' % operator_id"
    write(18,*) 'ARRAY_ID=', receiver_id , " %receiver 0=between nodes,1=on nodes"
    write(18,*) 'DT=', dt , "% disrete_time_intervall"
    write(18,*) 'NT=', nt , "% number_of_time_steps"
    write(18,*) 'L=', Length , "% Length_of_the_model"
    write(18,*) 'V=', min_v , "% velocity_of_the_model"
    write(18,*) 'TDEST=', tmax , "% total_simulated_time"
    write(18,*) 'DX=', dx_mean , "% mean_dx"
    write(18,*) 'NX=', ng , "% total_number_of_global_points"
    write(18,*) 'NUMRECS=', rec_num , "% number_of_receivers"
    write(18,*) 'SRCIXG=', source_ix , "% global_node_index_of_source_position"

  if (present(de)) then
    write(18,*) 'DE=', de , "% Length_of_the_elements"
  endif

    close(18)


END SUBROUTINE write_setup

